/*
-------------------------------------------------------------------------------
    This file is a part of the VST Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: vpgfile_test.cpp
  Author: Francois Mazen
  Date: 28/06/16 20:47
  Description: a test class for the VpgFile class
-------------------------------------------------------------------------------
*/

#include "../../vpgfile.h"
#include "../../vpgversion.h"
#include "../../vstlistmodel.h"

#include <QtXml/QDomDocument>

#include <QtTest>

class VpgFileTest : public QObject
{
    Q_OBJECT

public:
    VpgFileTest();

private Q_SLOTS:

    void initTestCase();
    void cleanupTestCase();

    void init();
    void cleanup();

    void testWriteFileErrorCodes();
    void testWriteFile();

    void testReadFileErrorCodes();
    void testReadFile();

    void testWriteThenReadFile();

private:
    VSTListModel* vstListModel;

    QFileInfo testVpgFile;
    QFileInfo validVPGFile;
    QFileInfo testReadWriteVpgFile;
    QFileInfo nonXMLTestFile;
    QFileInfo nonVPGTestFile;
    QFileInfo nonVstTestFile;
    QFileInfo invalidXMlElementTestFile;
};

//-----------------------------------------------------------------------------
VpgFileTest::VpgFileTest() :
    vstListModel(NULL),
    testVpgFile("testVpgFile.vpg"),
    validVPGFile("validVPGFile"),
    testReadWriteVpgFile("testReadWriteVpgFile.vpg"),
    nonXMLTestFile("nonXMLTestFile"),
    nonVPGTestFile("nonVPGTestFile"),
    nonVstTestFile("nonVstTestFile"),
    invalidXMlElementTestFile("invalidXMlElementTestFile")
{
    this->testVpgFile.setCaching(false);
    this->validVPGFile.setCaching(false);
    this->testReadWriteVpgFile.setCaching(false);
    this->nonXMLTestFile.setCaching(false);
    this->nonVPGTestFile.setCaching(false);
    this->nonVstTestFile.setCaching(false);
    this->invalidXMlElementTestFile.setCaching(false);
}

//-----------------------------------------------------------------------------
void VpgFileTest::initTestCase()
{
    this->vstListModel = new VSTListModel(NULL);
}

//-----------------------------------------------------------------------------
void VpgFileTest::cleanupTestCase()
{
    delete this->vstListModel;
    this->vstListModel = NULL;

    // Remove all written files during tests.
    if(this->testVpgFile.exists())
    {
        QFile::remove(this->testVpgFile.absoluteFilePath());
    }
    if(this->validVPGFile.exists())
    {
        QFile::remove(this->validVPGFile.absoluteFilePath());
    }
    if(this->testReadWriteVpgFile.exists())
    {
        QFile::remove(this->testReadWriteVpgFile.absoluteFilePath());
    }
    if(this->nonXMLTestFile.exists())
    {
        QFile::remove(this->nonXMLTestFile.absoluteFilePath());
    }
    if(this->nonVPGTestFile.exists())
    {
        QFile::remove(this->nonVPGTestFile.absoluteFilePath());
    }
    if(this->nonVstTestFile.exists())
    {
        QFile::remove(this->nonVstTestFile.absoluteFilePath());
    }
    if(this->invalidXMlElementTestFile.exists())
    {
        QFile::remove(this->invalidXMlElementTestFile.absoluteFilePath());
    }
}

//-----------------------------------------------------------------------------
void VpgFileTest::init()
{
    this->vstListModel->clearList();

    FxHeaderStruct* fxHeader1 = new FxHeaderStruct;
    fxHeader1->fxID = "ABCD";
    fxHeader1->fxPath = "a/path/to/file1";
    fxHeader1->fxVersion = 1234;
    fxHeader1->name = "Plug1";
    fxHeader1->nParams = 5;
    fxHeader1->useChunk = false;
    this->vstListModel->addPluginHeader(fxHeader1);

    FxHeaderStruct* fxHeader2 = new FxHeaderStruct;
    fxHeader2->fxID = "EFGH";
    fxHeader2->fxPath = "a/path/to/file2";
    fxHeader2->fxVersion = 5678;
    fxHeader2->name = "Plug2";
    fxHeader2->nParams = 42;
    fxHeader2->useChunk = true;
    this->vstListModel->addPluginHeader(fxHeader2);
}

//-----------------------------------------------------------------------------
void VpgFileTest::cleanup()
{
    vstListModel->clearList();
}

//-----------------------------------------------------------------------------
void VpgFileTest::testWriteFile()
{
    VpgFile::ERROR_CODE errorCode = VpgFile::WriteFile("testVpgFile.vpg", *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_NO_ERROR);

    QFile vpgFile("testVpgFile.vpg");
    QVERIFY(vpgFile.open(QIODevice::ReadOnly));
    QDomDocument doc;
    QVERIFY(doc.setContent(&vpgFile));
    vpgFile.close();

    QDomElement root = doc.documentElement();
    QCOMPARE(root.tagName(), QString("vpg_vst_list"));
    QCOMPARE(root.attribute("major_number").toUInt(), VpgVersion::getCurrentVersion().getMajorNumber());
    QCOMPARE(root.attribute("minor_number").toUInt(), VpgVersion::getCurrentVersion().getMinorNumber());
    QCOMPARE(root.attribute("revision_number").toUInt(), VpgVersion::getCurrentVersion().getRevisionNumber());

    QCOMPARE(root.childNodes().size(), 2);

    QDomNode node1 = root.childNodes().at(0);
    QVERIFY(!node1.isNull());
    QVERIFY(node1.isElement());
    QDomElement element1 = node1.toElement();
    QCOMPARE(element1.tagName(), QString("vst"));
    QCOMPARE(element1.attributes().size(), 6);
    QCOMPARE(element1.attribute("fx_path"), QString("a/path/to/file1"));
    QCOMPARE(element1.attribute("fx_id"), QString("ABCD"));
    QCOMPARE(element1.attribute("chunk"), QString("0"));
    QCOMPARE(element1.attribute("nb_param"), QString("5"));
    QCOMPARE(element1.attribute("name"), QString("Plug1"));
    QCOMPARE(element1.attribute("fx_version"), QString("1234"));

    QDomNode node2 = root.childNodes().at(1);
    QVERIFY(!node2.isNull());
    QVERIFY(node2.isElement());
    QDomElement element2 = node2.toElement();
    QCOMPARE(element2.tagName(), QString("vst"));
    QCOMPARE(element2.attributes().size(), 6);
    QCOMPARE(element2.attribute("fx_path"), QString("a/path/to/file2"));
    QCOMPARE(element2.attribute("fx_id"), QString("EFGH"));
    QCOMPARE(element2.attribute("chunk"), QString("1"));
    QCOMPARE(element2.attribute("nb_param"), QString("42"));
    QCOMPARE(element2.attribute("name"), QString("Plug2"));
    QCOMPARE(element2.attribute("fx_version"), QString("5678"));

}

//-----------------------------------------------------------------------------
void VpgFileTest::testWriteFileErrorCodes()
{
    // ERROR_COULD_NOT_OPEN_FILE: Test with an invalid filename.
    VpgFile::ERROR_CODE errorCode = VpgFile::WriteFile("", *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_COULD_NOT_OPEN_FILE);
}

//-----------------------------------------------------------------------------
void VpgFileTest::testReadFile()
{
    QFile vpgFile("validVPGFile");
    QVERIFY(vpgFile.open(QIODevice::WriteOnly));
    vpgFile.write("<vpg_vst_list major_number=\"0\" minor_number=\"2\" revision_number=\"6\">");
    vpgFile.write("<vst fx_path=\"\" fx_id=\"gyAm\" chunk=\"0\" nb_param=\"8\" name=\"GComp2\" fx_version=\"1100\"/>");
    vpgFile.write("<vst fx_path=\"\" fx_id=\"gyAh\" chunk=\"0\" nb_param=\"3\" name=\"GHi\" fx_version=\"100\"/>");
    vpgFile.write("<vst fx_path=\"\" fx_id=\"gyAd\" chunk=\"0\" nb_param=\"4\" name=\"GClip\" fx_version=\"10\"/>");
    vpgFile.write("<vst fx_path=\"path/to/plug1\" fx_id=\"Tf40\" chunk=\"1\" nb_param=\"112\" name=\"tunefish4\" fx_version=\"100\"/>");
    vpgFile.write("<vst fx_path=\"path/to/plug2\" fx_id=\"LGPd\" chunk=\"1\" nb_param=\"36\" name=\"Digits\" fx_version=\"0\"/>");
    vpgFile.write("</vpg_vst_list>");
    vpgFile.close();

    VpgFile::ERROR_CODE errorCode = VpgFile::ReadFile(vpgFile.fileName(), *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_NO_ERROR);

    QCOMPARE(this->vstListModel->getPluginHeaders().size(), 5);

    FxHeaderStruct* fxHeader = this->vstListModel->getPluginHeaderAtIndex(0);
    QVERIFY(fxHeader != NULL);
    QCOMPARE(fxHeader->fxPath, QString(""));
    QCOMPARE(fxHeader->fxID, QString("gyAm"));
    QCOMPARE(fxHeader->useChunk, false);
    QCOMPARE(fxHeader->nParams, 8);
    QCOMPARE(fxHeader->name, QString("GComp2"));
    QCOMPARE(fxHeader->fxVersion, 1100);

    fxHeader = this->vstListModel->getPluginHeaderAtIndex(1);
    QVERIFY(fxHeader != NULL);
    QCOMPARE(fxHeader->fxPath, QString(""));
    QCOMPARE(fxHeader->fxID, QString("gyAh"));
    QCOMPARE(fxHeader->useChunk, false);
    QCOMPARE(fxHeader->nParams, 3);
    QCOMPARE(fxHeader->name, QString("GHi"));
    QCOMPARE(fxHeader->fxVersion, 100);

    fxHeader = this->vstListModel->getPluginHeaderAtIndex(2);
    QVERIFY(fxHeader != NULL);
    QCOMPARE(fxHeader->fxPath, QString(""));
    QCOMPARE(fxHeader->fxID, QString("gyAd"));
    QCOMPARE(fxHeader->useChunk, false);
    QCOMPARE(fxHeader->nParams, 4);
    QCOMPARE(fxHeader->name, QString("GClip"));
    QCOMPARE(fxHeader->fxVersion, 10);

    fxHeader = this->vstListModel->getPluginHeaderAtIndex(3);
    QVERIFY(fxHeader != NULL);
    QCOMPARE(fxHeader->fxPath, QString("path/to/plug1"));
    QCOMPARE(fxHeader->fxID, QString("Tf40"));
    QCOMPARE(fxHeader->useChunk, true);
    QCOMPARE(fxHeader->nParams, 112);
    QCOMPARE(fxHeader->name, QString("tunefish4"));
    QCOMPARE(fxHeader->fxVersion, 100);

    fxHeader = this->vstListModel->getPluginHeaderAtIndex(4);
    QVERIFY(fxHeader != NULL);
    QCOMPARE(fxHeader->fxPath, QString("path/to/plug2"));
    QCOMPARE(fxHeader->fxID, QString("LGPd"));
    QCOMPARE(fxHeader->useChunk, true);
    QCOMPARE(fxHeader->nParams, 36);
    QCOMPARE(fxHeader->name, QString("Digits"));
    QCOMPARE(fxHeader->fxVersion, 0);
}

//-----------------------------------------------------------------------------
void VpgFileTest::testWriteThenReadFile()
{
    VpgFile::ERROR_CODE errorCode = VpgFile::WriteFile("testReadWriteVpgFile.vpg", *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_NO_ERROR);

    VSTListModel newVSTListModel(NULL);
    errorCode = VpgFile::ReadFile("testReadWriteVpgFile.vpg", newVSTListModel);
    QCOMPARE(errorCode, VpgFile::ERROR_NO_ERROR);

    QCOMPARE(newVSTListModel.getPluginHeaders().size(), this->vstListModel->getPluginHeaders().size());
    QCOMPARE(newVSTListModel.getPluginHeaders().size(), 2);

    for(int i=0; i<2; i++)
    {
        QCOMPARE(newVSTListModel.getPluginHeaderAtIndex(i)->fxID, this->vstListModel->getPluginHeaderAtIndex(i)->fxID);
        QCOMPARE(newVSTListModel.getPluginHeaderAtIndex(i)->fxPath, this->vstListModel->getPluginHeaderAtIndex(i)->fxPath);
        QCOMPARE(newVSTListModel.getPluginHeaderAtIndex(i)->fxVersion, this->vstListModel->getPluginHeaderAtIndex(i)->fxVersion);
        QCOMPARE(newVSTListModel.getPluginHeaderAtIndex(i)->name, this->vstListModel->getPluginHeaderAtIndex(i)->name);
        QCOMPARE(newVSTListModel.getPluginHeaderAtIndex(i)->nParams, this->vstListModel->getPluginHeaderAtIndex(i)->nParams);
        QCOMPARE(newVSTListModel.getPluginHeaderAtIndex(i)->useChunk, this->vstListModel->getPluginHeaderAtIndex(i)->useChunk);
    }
}

//-----------------------------------------------------------------------------
void VpgFileTest::testReadFileErrorCodes()
{
    // ERROR_COULD_NOT_OPEN_FILE
    VpgFile::ERROR_CODE errorCode = VpgFile::ReadFile("total_invalid_file", *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_COULD_NOT_OPEN_FILE);

    // ERROR_COULD_NOT_PARSE_FILE
    QFile invalidFile("nonXMLTestFile");
    QVERIFY(invalidFile.open(QIODevice::WriteOnly));
    invalidFile.write("this is not a vpg file!");
    invalidFile.close();
    errorCode = VpgFile::ReadFile(invalidFile.fileName(), *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_COULD_NOT_PARSE_FILE);

    // ERROR_INVALID_VPG_FILE
    QFile invalidVPGFile("nonVPGTestFile");
    QVERIFY(invalidVPGFile.open(QIODevice::WriteOnly));
    invalidVPGFile.write("<root><item/><item/></root>");
    invalidVPGFile.close();
    errorCode = VpgFile::ReadFile(invalidVPGFile.fileName(), *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_INVALID_VPG_FILE);

    // ERROR_UNKNOWN_XML_TAG
    QFile invalidVstFile("nonVstTestFile");
    QVERIFY(invalidVstFile.open(QIODevice::WriteOnly));
    invalidVstFile.write("<vpg_vst_list>");
    invalidVstFile.write("  <vst fx_path=\"\" fx_id=\"gyAm\" chunk=\"0\" nb_param=\"8\" name=\"GComp2\" fx_version=\"1100\"/>");
    invalidVstFile.write("  <invalid_tag/>");
    invalidVstFile.write("</vpg_vst_list>");
    invalidVstFile.close();
    errorCode = VpgFile::ReadFile(invalidVstFile.fileName(), *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_UNKNOWN_XML_TAG);

    // ERROR_INVALID_XML_ELEMENT
    QFile invalidXMLElementFile("invalidXMlElementTestFile");
    QVERIFY(invalidXMLElementFile.open(QIODevice::WriteOnly));
    invalidXMLElementFile.write("<vpg_vst_list>");
    invalidXMLElementFile.write("  <!-- This is a comment -->");
    invalidXMLElementFile.write("</vpg_vst_list>");
    invalidXMLElementFile.close();
    errorCode = VpgFile::ReadFile(invalidXMLElementFile.fileName(), *(this->vstListModel));
    QCOMPARE(errorCode, VpgFile::ERROR_INVALID_XML_ELEMENT);
}

QTEST_APPLESS_MAIN(VpgFileTest)

#include "vpgfile_test.moc"
